@@ -9,10 +9,14 @@ class youtube(utils.video):
 
 class YouTube(utils.Video):
     _node = youtube
+    _thumbnail_url = "https://i3.ytimg.com/vi/{}/maxresdefault.jpg"
 
 
 def visit_youtube_node(self, node):
-    return utils.visit_video_node(self, node, platform_url="https://www.youtube.com/embed/")
+    return utils.visit_video_node(self, node,
+                                  platform_url="https://www.youtube.com/embed/",
+                                  platform_url_privacy="https://www.youtube-nocookie.com/embed/"
+                                  )
 
 
 def visit_youtube_node_latex(self, node):
